from django.contrib.auth.models import User
from django.db.models import Model

from rules import predicate

from aleksis.apps.chronos.models import Room
from aleksis.core.models import Group, Person
from aleksis.core.util.predicates import has_global_perm, has_object_perm


@predicate
def has_timetable_perm(user: User, obj: Model) -> bool:
    """Predicate which checks whether the user is allowed to access the requested timetable."""
    if type(obj) is Group:
        return (
            user.person.member_of.filter(id=obj.id).exists()
            or user.person.primary_group == obj
            or user.person.owner_of.filter(id=obj.id).exists()
            or has_global_perm("chronos.view_all_group_timetables")(user)
            or has_object_perm("core.view_group_timetable")(user, obj)
        )
    elif type(obj) is Person:
        return (
            user.person == obj
            or has_global_perm("chronos.view_all_person_timetables")(user)
            or has_object_perm("core.view_person_timetable")(user, obj)
        )
    elif type(obj) is Room:
        return has_global_perm("chronos.view_all_room_timetables")(user) or has_object_perm(
            "chronos.view_room_timetable"
        )(user, obj)
    else:
        return False
